/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package ZEN_IDL.be;

import java.io.*;
import java.util.*;
import ZEN_IDL.ast.*;
import ZEN_IDL.be.helperUtils.*;

/*********************************************************
* Takes care of compressing interfaceNames and method
* names. Currently uses only a counter, e.g. interface
* name one get no. 0, interface number two get 1. etc.
*
* @author Olav Haugan
*
* 14.10.01
*********************************************************/
public class MQCCompress
{
	private static int interfaceNo = 0;
	private static int methodNo = 0;
	private static Hashtable methodTbl = new Hashtable(4);
	private static Hashtable interfaceTbl = new Hashtable(4);

	/*********************************************************
	* Returns the compressed interface name
	*
	* @param String The name of the interface
	*
	* @return int The compressed name
	*********************************************************/
	public static int compressInterface(String name)
	{
		Integer no = (Integer) interfaceTbl.get(name);
		if(no == null)
		{
			interfaceTbl.put(name, new Integer(interfaceNo));
		 	no = new Integer(interfaceNo);
			interfaceNo++;
		}
		return no.intValue();
	}

	/*********************************************************
	* Returns the compressed method name
	*
	* @param String The name of the method
	*
	* @return int The compressed name
	*********************************************************/
	public static int compressMethod(String name)
	{
		Integer no = (Integer) methodTbl.get(name);
		if(no == null)
		{
			methodTbl.put(name, new Integer(methodNo));
		 	no = new Integer(methodNo);
		 	methodNo++;
		}
		return no.intValue();
	}

	/*********************************************************
	* Deletes all mappings of interfaces and methods to 
	* compressed names.
	*********************************************************/
	public static void clear()
	{
		methodTbl.clear();
		methodNo = 0;
		interfaceTbl.clear();
		interfaceNo = 0;
	}
}
