/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package ZEN_IDL.be;

import ZEN_IDL.ast.*;
import ZEN_IDL.be.symbolTableUtils.*;
import ZEN_IDL.be.helperUtils.*;
import util.*;

import java.io.*;
import java.util.*;

/*********************************************************
* Generates stubs and skeleton library marshalling and 
* unmarshalling code.
*
* @author Olav Haugan
*
* 14.10.01
*********************************************************/
public class MQCLibMarshalHelper extends MQCAbstractMarshalHelper
{
	/*********************************************************
	* Constructor. Initializes variables.
	* 
	* @param Hashtable A method mapping object
	*        
	*********************************************************/
	public MQCLibMarshalHelper(MQCConfig config)
	{
		super(config);
	}

	/*********************************************************
	* Initializes the marshaller.
	*********************************************************/
	public void init()
	{
	}
	
	/*********************************************************
	* Prints the marshalling code for 
	* either the client or server. 
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	* @param boolean true if code is for client, false
  *                if for server.
	*
	* @return int The size of buffer needed.
	*********************************************************/
	public int printMarshal( MQCPrinter pw, InterfaceMethod iMethod, boolean Client )
	{
		if(Client)
			return printClientMarshal(pw, iMethod );
		else
			return printServerMarshal(pw, iMethod );
	}

	/*********************************************************
	* Prints the marshalling code for 
	* either the client or server. 
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	* @param boolean true if code is for client, false
  *                if for server.
	*
	*********************************************************/
	public void printUnMarshal( MQCPrinter pw, InterfaceMethod iMethod, boolean Client )
	{
		if(Client)
			printClientUnMarshal(pw, iMethod );
		else
			printServerUnMarshal(pw, iMethod );
	}
	
	/*********************************************************
	* Prints the client marshalling code
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	*********************************************************/
	private int printClientMarshal(MQCPrinter pw, InterfaceMethod iMethod )
	{
		Enumeration e = iMethod.paramVector.elements();
		InterfaceMethodParam arg = null;
//		pw.setIndent(2);
				
		while( e.hasMoreElements() )
		{
			arg = (InterfaceMethodParam) e.nextElement();
			if( !arg.paramInOut.equals("out") )
			{
				if(codeGenHelper.isPrimitiveType(arg.paramType))
				{
					pw.print("_output.write_" + codeGenHelper.mapIDL2RWMap( arg.paramType ));
					pw.nprint("( " + arg.paramName );

					if(arg.paramInOut.equals("inout"))
						pw.nprint(".value");

					pw.nprintln(" );");
				}
				else
				{
					pw.print(arg.paramType + "Helper.write( _output, " + arg.paramName );

					if(arg.paramInOut.equals("inout"))
						pw.nprint(".value");

					pw.nprintln(" );");
				}
			}
		}
		return 0;
	}


	/*********************************************************
	* Prints the server marshalling code
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	*********************************************************/
	private int printServerMarshal(MQCPrinter pw, InterfaceMethod iMethod )
	{
		Enumeration e = iMethod.paramVector.elements();
		InterfaceMethodParam arg = null;
//		pw.setIndent(3);
		
		if(!iMethod.returnType.equals("void"))
		{
			if(codeGenHelper.isPrimitiveType(iMethod.returnType))
				pw.println("_output.write_" + codeGenHelper.mapIDL2RWMap( iMethod.returnType ) + "(_result);");
			else
				pw.println( iMethod.returnType + "Helper.write(_output, _result);");
		}

		while( e.hasMoreElements() )
		{
			arg = (InterfaceMethodParam) e.nextElement();
			if( !arg.paramInOut.equals("in") )
			{
				if(codeGenHelper.isPrimitiveType(arg.paramType))
				{
					pw.print("_output.write_" + codeGenHelper.mapIDL2RWMap( arg.paramType ));
					pw.nprintln("( " + arg.paramName + ".value );");
				}
				else
				{
					pw.println(arg.paramType + "Helper.write( _output, " + arg.paramName + ".value );");
				}
			}
		}
		return 0;
	}

	/*********************************************************
	* Prints the client unmarshalling code
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	*********************************************************/
	private void printClientUnMarshal(MQCPrinter pw, InterfaceMethod iMethod )
	{
//		pw.setIndent(2);
		if(!iMethod.returnType.equals("void"))
		{
			if(codeGenHelper.isPrimitiveType(iMethod.returnType))
			{
				pw.print("_result = _input.read_");
				pw.nprintln( codeGenHelper.mapIDL2RWMap(iMethod.returnType) + "();");
			}
			else
				pw.println("_result = " + iMethod.returnType + "Helper.read(_input);");
		}

		Enumeration e = iMethod.paramVector.elements();
		InterfaceMethodParam arg = null;
		while( e.hasMoreElements() )
		{
			arg = (InterfaceMethodParam) e.nextElement();
			if( !arg.paramInOut.equals("in") )
			{
				if(codeGenHelper.isPrimitiveType(arg.paramType))
				{
					pw.print(arg.paramName + ".value = _input.read_");
					pw.nprintln( codeGenHelper.mapIDL2RWMap( arg.paramType ) + "();" );
				}
				else
				{
					pw.println(arg.paramName + ".value = " + arg.paramType + "Helper.read(_input);");
				}
			}
		}
	}

	/*********************************************************
	* Prints the server unmarshalling code
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	*********************************************************/
	private void printServerUnMarshal(MQCPrinter pw, InterfaceMethod iMethod )
	{
		Enumeration e = iMethod.paramVector.elements();
		InterfaceMethodParam arg = null;
//		pw.setIndent(3);
		while( e.hasMoreElements() )
		{
			arg = (InterfaceMethodParam) e.nextElement();
			if( arg.paramInOut.equals("in") )
			{
				if(codeGenHelper.isPrimitiveType(arg.paramType))
					pw.println(codeGenHelper.mapIDL2Java( arg.paramType ) + " " + arg.paramName + " = _input.read_" + codeGenHelper.mapIDL2RWMap( arg.paramType ) + "();");
				else
					pw.println(codeGenHelper.mapIDL2Java( arg.paramType ) + " " + arg.paramName + " = " + arg.paramType + "Helper.read(_input);");
			}
			else
			{
				pw.print(codeGenHelper.mapIDL2JavaHolder( arg.paramType ) );
				pw.nprintln( " " + arg.paramName + " = new " + codeGenHelper.mapIDL2JavaHolder( arg.paramType ) + "();");				
				if(arg.paramInOut.equals("inout"))
				{
					if(codeGenHelper.isPrimitiveType(arg.paramType))
						pw.println(arg.paramName + ".value = _input.read_" + codeGenHelper.mapIDL2RWMap(arg.paramType) +  "();");
					else
						pw.println(arg.paramName + ".value = " + arg.paramType + "Helper.read(_input);");
				}
			}
		}
	}
}
