/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package ZEN_IDL.be;

import java.util.*;
import java.io.*;
import ZEN_IDL.ast.*;
import ZEN_IDL.be.helperUtils.*;
import ZEN_IDL.be.symbolTableUtils.*;

public class VisitorState
{
	public Stack stkPackages;		//stack of packages. mapped from modules.
	private int intCurrentNodeType;		//based on IDLParserTreeConstants
	private String strNodeName;
	private boolean inInterface;		//true if inside interface. used for constants code
	private PrintWriter out;
	private boolean errorOccured;
        
        //-----------current parameter ----------------
        private InterfaceMethodParam currentParam;
            
        public void initializeCurrentParam()
        {
            currentParam = new InterfaceMethodParam();
        }

        public void setCurrentParamInOut( String inOut )
        {
            currentParam.paramInOut = inOut;
        }

        public void setCurrentParamType( String paramType )
        {
	    currentParam.paramType = paramType;
        }
   
        public void setCurrentParamName( String paramName )
        {
            currentParam.paramName = paramName;
        }
 
        public void addCurrentParamToCurrentMethod()
        {
            currentMethod.addParam( currentParam );
        }
 
        //-----------current interface method ---------
        private InterfaceMethod currentMethod;
        
        public void initializeCurrentMethod()
        {
            currentMethod = new InterfaceMethod();
        }
        
        public void setCurrentMethodAsOneway()
        {
            currentMethod.isOneWay = true;
        } 

        public void setCurrentMethodReturnType( String returnType )
        {
            currentMethod.returnType = returnType;
        }
       
        public void setCurrentMethodName( String name )
        {
            currentMethod.name = name; 
	    currentMethod.overrideGiopName = name;
        }

	public void setCurrentMethodGIOPName( String name )
	{
	    currentMethod.overrideGiopName = name;
	}

        public void addParam( Object param )
        {
            currentMethod.paramVector.add( param );
        }

        public void addException( Object exception )
        {
            currentMethod.exceptionVector.add( exception );
        }
  
        public void addContextExprTo( Object contextExpr )
        {
            currentMethod.contextExprVector.add( contextExpr );
        }  

        public void addCurrentMethod()
        {
            addInterfaceMethod( currentMethod );
        }

        //-----------interface methods-------
        private Vector interfaceMethodsVector;
        
        public void initializeMethodsVector()
        {
            interfaceMethodsVector = new Vector();
        }
      
        public void addInterfaceMethod( Object method )
        {
            interfaceMethodsVector.add( method );
	    System.out.println( "method added. total: " + interfaceMethodsVector.size() );
        }

        public Vector getInterfaceMethodsVector()
        {
            return interfaceMethodsVector;
        }
	//------------Exception info--------
	private String reasonType;
	private String reasonCode;
	
	public void addExceptionReasonType(String type )
        {
	      reasonType = type;
        }
	public void addExceptionReasonCode(String code )
	{
	    reasonCode = code; 
	}
	public String getExceptionReasonType()
 	{
	   return reasonType;
	}
	public String getExceptionReasonCode()
        {
	    return reasonCode;
	}       
	
        //-----------Union vectors----------
        private Vector unionTypeSpecVector;
        private Vector unionDeclaratorVector;

        public void initializeUnionVectors()
        {
            unionTypeSpecVector = new Vector();
            unionDeclaratorVector = new Vector();
        } 

        public void addUnionTypeSpec( Object typeSpec )
        {
            unionTypeSpecVector.add( typeSpec );
        }

        public void addUnionDeclarator( Object declarator ) 	
        {
            unionDeclaratorVector.add( declarator );
        }

        public Vector getUnionTypeSpecVector()
        {
            return unionTypeSpecVector;
        }

        public Vector getUnionDeclaratorVector()
        {
            return unionDeclaratorVector;
        }
        
	//-----------typedef info------------
	private String typedefType;
	private TypedefDeclarator typedefDeclarator;

	public void setTypedefType( String type)
	{
	    typedefType = type;
	}
	public String getTypedefType()
	{
	    return typedefType;
	}
        public void setTypedefDeclarator( TypedefDeclarator td )
	{
	    typedefDeclarator = td;
	}
	public TypedefDeclarator getTypedefDeclarator()
	{
	    return typedefDeclarator;
	}
        //-----------struct vectors----------
        private Vector structTypeSpecVector;
        private Vector structDeclaratorVector;

        public void initializeStructVectors()
        {
            structTypeSpecVector = new Vector();
            structDeclaratorVector = new Vector();
        }

        public void addStructTypeSpec( Object typeSpec )
        {
            structTypeSpecVector.add( typeSpec );
        }

        public void addStructDeclarator( Object declarator ) 	
        {
            structDeclaratorVector.add( declarator );
        }

        public Vector getStructTypeSpecVector()
        {
            return structTypeSpecVector;
        }

        public Vector getStructDeclaratorVector()
        {
            return structDeclaratorVector;
        }

        //--------- array info in structs ------
        private String arrayBracketsAndSize;
        public void initializeArrayBracketsAndSize()
        {
            arrayBracketsAndSize = "";
        }
        public void addBracketAndSize( String bracketAndSize )
        {
            arrayBracketsAndSize += bracketAndSize;
        }
        public String getArrayBracketsAndSize()
        {
            return arrayBracketsAndSize;
        }
	//---------_interface stuff----------
	class InterfaceProperties{
		InterfaceProperties( String name , boolean isAbstract , Vector inheritenceSpec )
		{
			this.name = name;
			this.isAbstract = isAbstract;
			this.inheritenceSpec = inheritenceSpec;
		}
		public String name;
		public boolean isAbstract;
		public Vector inheritenceSpec;
	}
	private Hashtable interfaceList;	
	//-----------------------------------


    	public VisitorState()
	{
		stkPackages = new Stack();
		interfaceList = new Hashtable();
		errorOccured=false;
	}
	public String getPackage()
	{
		Enumeration packageList = stkPackages.elements();
		StringBuffer returnValue = new StringBuffer();

        	while( packageList.hasMoreElements() )
		{
		    returnValue.append( packageList.nextElement() );
		    returnValue.append( "." );
		}

		if( returnValue.toString().trim().equals( "" ) )
			return "";
		returnValue.setCharAt( returnValue.length()-1 , ' ' );
		return returnValue.toString().trim(); 
	}
	
    	public String getPackageString()
    	{	
        	return new String( "package " + getPackage() + ";" );
    	}

    	public String getPackagePath()
    	{
        	Enumeration packageList = stkPackages.elements();
		StringBuffer returnValue = new StringBuffer();

        	while( packageList.hasMoreElements() )
		{
		    returnValue.append( (String) packageList.nextElement() );
		    returnValue.append( File.separator );
		}
        	return new String( returnValue );
    	}

	public Vector getPackageVector(){
		Enumeration packageList = stkPackages.elements();
		Vector returnValue = new Vector();

        	while( packageList.hasMoreElements() )
		{
		    returnValue.add( (String)packageList.nextElement() );
		}
        	return returnValue;
	}
	
	public void setCurrentNodeType( int nodeTypeID )
	{
		intCurrentNodeType = nodeTypeID;
	}
	public int getCurrentNodeType()
	{
		return intCurrentNodeType;
	}
	public void setNodeName( String name )
	{
		strNodeName=name;
	}
	public String getNodeName()
	{
		return strNodeName;
	}
	public void setInInterface( boolean value )
	{
		inInterface = value;
	}
	public boolean getInInterface()
	{
		return inInterface;
	}
	public void setWriter( PrintWriter newOut )
	{
		out = newOut;
	}
	public PrintWriter getWriter()
	{
		return out;
	}

	//----------interface stuff-----------
	
//	public void addInterfaceToList( String name , boolean isAbstract , Vector inheritenceSpec )
//	{
//		InterfaceProperties iface = new InterfaceProperties( name , isAbstract , inheritenceSpec );
//		interfaceList.put( name , iface );
//	}

	public boolean isInterfaceAbstract( String name ,String mpackage )
	{
		return SymbolTable.getSymbol( name , mpackage ).getIsAbstract();
	}

	public InterfaceInheritence getInterfaceInheritenceSpec( String name , String mpackage )
	{
		return SymbolTable.getSymbol( name , mpackage ).getInheritenceSpec();
	}
	
	//-----------error reporting-----------

	public void setErrorOccured( boolean value ){ errorOccured = value; }
	public boolean getErrorOccured( ){ return errorOccured; }
}
