#include <stdio.h>
#include <string.h>

// NOTE: The get_cpu_clock_speed code was taken/based upon an example found at
// www.newriders.com in a pdf file discussing the /proc file system.

/*
 * Returns the clock speed of the system's CPU in MHz, as reported 
 * by /proc/cpuinfo. On a multiprocessor machine, returns the speed
 * of the first CPU. On error returns zero. 
 */
double get_cpu_clock_speed ()
{
   FILE *fp;
   char buffer[1024];
   size_t bytes_read;
   char* match;
   double clock_speed;

   /* Read the entire contents of /proc/cpuinfo into the buffer. */
   fp = fopen ("/proc/cpuinfo", "r");
   bytes_read = fread (buffer, 1, sizeof (buffer), fp);
   fclose (fp);

   /* Bail if read failed or if buffer is not big enough. */
   if (bytes_read == 0 || bytes_read == sizeof (buffer))
      return 0;

   /* NUL-terminate the text. */
   buffer[bytes_read] = '\0';

   /* Locate the line that starts with 'cpu MHz'. */
   match = strstr (buffer, "cpu MHz");
   if (match == NULL)
      return 0;

   /* Parse the line to extract the clock speed. */
   sscanf (match, "cpu MHz : %lf", &clock_speed);
   return clock_speed;
}


static int    _ticksPerSecond = 1;
static double _cpuMHz = 1;
static double _tscConv = 1;


void timer_init (int ticksPerSecond)
{
   _ticksPerSecond = ticksPerSecond;
   _cpuMHz         = get_cpu_clock_speed();
   _tscConv        = _ticksPerSecond / (1000000.*_cpuMHz);
}


unsigned long long timer_currentTime ()
{
   unsigned long long int x;
   double y;

   __asm__ volatile(".byte 0x0f,0x31" : "=A" (x));

   y = (double) x * _tscConv;

   return (unsigned long long) y;
}


double timer_getTimeResolution ()
{
   return _ticksPerSecond;
}
