/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package mqc.protocols;

import mqc.Config;

public class Message
{

   public static int getHdrSize( byte[] buf )
   {
      if ( buf.length >= 4 )
      {
         // GIOP
         if (     buf[0] == GIOPHeader.GMAGIC 
               && buf[1] == GIOPHeader.IMAGIC 
               && buf[2] == GIOPHeader.OMAGIC 
               && buf[3] == GIOPHeader.PMAGIC )
         {
            return 12;
         }

         // MQCIOP
         if (     buf[0] == MQCIOPHeader.MMAGIC 
               && buf[1] == MQCIOPHeader.QMAGIC 
               && buf[2] == MQCIOPHeader.CMAGIC )
         {
            return 12;
         }

         // GIOPLite
         return 8;
      }
      throw new RuntimeException( "Buffer too small." );
   }

   /**
    * Returns the current header size
    * @return The size of the message
    */
   public static int getMsgSize(byte [] buf, boolean bigEndian)
   {
      if (     (buf[0] == GIOPHeader.GMAGIC) 
            && (buf[1] == GIOPHeader.IMAGIC) 
            && (buf[2] == GIOPHeader.OMAGIC) 
            && (buf[3] == GIOPHeader.PMAGIC) )
      {
         return GIOPHeader.getMsgSize(buf, bigEndian);
      }
      if (     (buf[0] == MQCIOPHeader.MMAGIC) 
            && (buf[1] == MQCIOPHeader.QMAGIC) 
            && (buf[2] == MQCIOPHeader.CMAGIC))
      {
         return MQCIOPHeader.getMsgSize(buf, bigEndian);
      }
      return GIOPLiteHeader.getMsgSize(buf, bigEndian);
   }

   public static boolean isBigEndian( byte[] buf )
   {
      if ( buf.length >= 4 )
      {
         // GIOP
         if (     buf[0] == GIOPHeader.GMAGIC 
               && buf[1] == GIOPHeader.IMAGIC 
               && buf[2] == GIOPHeader.OMAGIC 
               && buf[3] == GIOPHeader.PMAGIC )
         {
            return (buf[6] & (byte)0x01) == 0;
         }

         // MQCIOP
         if (     buf[0] == MQCIOPHeader.MMAGIC 
               && buf[1] == MQCIOPHeader.QMAGIC 
               && buf[2] == MQCIOPHeader.CMAGIC )
         {
            return (buf[3] & (byte)0x01) == 0;
         }

         // GIOPLite
         return !(Config.FORCE_ENDIANESS && !Config.BIG_ENDIAN);
      }
      throw new RuntimeException( "Buffer too small." );
   }
}
