/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

include(`Config_macros.m4')

package mqc.transports;

import mqc.*;
import mqc.ior.*;
import mqc.holders.*;
import mqc.transports.*;
import java.io.*;
import java.util.*;
__J2SE__(import java.net.*;)
__CLDC__(import javax.microedition.io.*;)

/**
 * UDP client transport base class.
 *
 * @author Thor Egil Skaug     initial
 * @version 1.00
 */

public abstract class UDPGenericTransport extends Transport
{
   protected String m_strServerName;                                          //the host name of the server
   protected int m_nServerPort;                                               //the port on which the server accepts connects packets
   protected byte[] m_recvBuf = null;
   protected byte[] m_sendBuf = null;
   protected int m_nMessageSize=0;
   protected int m_nHeaderSize=0;
   protected int m_nProtocol=0;
   __J2SE__(
   protected DatagramSocket m_serverSocket;
   protected int m_nServerSessionPort;                                        //the server port used for the session
   )
   __CLDC__(
   protected DatagramConnection m_serverSocket;
   protected String m_strServerSessionURL;                                    //the GCF url for the server this session
   )
   __SECURITY__(
   protected int m_nPadOffset=0;                                              //offset of encryption alignment padding
   )
   __SECURITY_MAC__(
   protected int m_nMacOffset=0;                                              //offset of mac
   )
   __SECURITY_MD__(
   __SECURITY_MD_NOT_HMAC__(
   protected int m_nMdOffset=0;                                               //offset of md
   )
   )
   
   public UDPGenericTransport(mqc.Delegate delegate)
   {
      super(delegate);
      Profile profile = delegate.ior.getProfile();
      m_nProtocol = delegate.protocol;
      TaggedProfile tagProfile = null;
      if ( profile.getTaggedProfile(TaggedProfile.UIOP) != null)
      {
         tagProfile = profile.getTaggedProfile(TaggedProfile.UIOP);
      }
      else if (profile.getTaggedProfile(TaggedProfile.UIOPLITE) != null)
      {
         tagProfile = profile.getTaggedProfile(TaggedProfile.UIOPLITE);
      }
      else if (profile.getTaggedProfile(TaggedProfile.UMQCIOP) != null)
      {
         tagProfile = profile.getTaggedProfile(TaggedProfile.UMQCIOP);
      }
      else
      {
         System.err.println ("**Error: no transport layer found");
      }
      m_strServerName = tagProfile.host;
      m_nServerPort = tagProfile.port;

      __FIX_SIZE__(
      //in fixed size messages, the offsets are always the same
      m_nMessageSize = Config.intPacketSize;
      __SECURITY_MD__(
      __SECURITY_MD_NOT_HMAC__(
      m_nMdOffset = m_nMessageSize;                                           //do not add MD len if the MD is for
      m_nMessageSize += ORB.msgDigestLen;                                     //an HMAC (ie, no double counting)
      )
      )
      __SECURITY_MAC__(
      m_nMacOffset = m_nMessageSize;
      m_nMessageSize += ORB.msgMacLen;
      )
      __SECURITY__(
      m_nPadOffset = m_nMessageSize;                                          //sets the padding offset (no double padding!!)
      m_nMessageSize = ORB.cipherDecrypt.getOutputSize(m_nMessageSize);     //adds whatever padding is needed to the message size
      )
      m_recvBuf = new byte [m_nMessageSize];                                  //create the fixed size receive buffer
      m_sendBuf = new byte [m_nMessageSize];                                  //create the fixed size send buffer
      )
      __VAR_SIZE__(
      //for UDP we cannot have a growing receive buffer, so
      //if Config.intPacketSize is set to zero, we set this static
      m_recvBuf = new byte[__MAX_UDP_DATAGRAM_SIZE__];
      )   

   }

   protected void establishServerConnection(int nUDPType) throws IOException __J2SE__(, SocketException)
   {
      byte connBuf[] = new byte[1];
      connBuf[0] = (byte) nUDPType;

      //send a connect message to the server
      __J2SE__(
      m_serverSocket = new DatagramSocket();
      m_serverSocket.send(new DatagramPacket(connBuf, connBuf.length, InetAddress.getByName(m_strServerName), m_nServerPort));
      )
      __CLDC__(
      String strConnURL = "datagram://" + m_strServerName + ":" + Integer.toString(m_nServerPort);
      m_serverSocket = (DatagramConnection)Connector.open(strConnURL, Connector.READ_WRITE, false);
      Datagram connDatagram = m_serverSocket.newDatagram(connBuf, connBuf.length);
      m_serverSocket.send(connDatagram);          
      )
      
      //wait for a ACK/NACK from the server
      __J2SE__(
      DatagramPacket packet = new DatagramPacket(connBuf, connBuf.length);
      m_serverSocket.receive(packet);
      m_nServerSessionPort = packet.getPort();
      )
      __CLDC__(
      m_serverSocket.receive(connDatagram);
      m_strServerSessionURL = connDatagram.getAddress();
      )
      
      //TBI: verify that the packet contains an ACK
      
      //connect to the new port specified by the server
      __J2SE__(
      //TODO we don't really need this
//      m_serverSocket.disconnect();
//      m_serverSocket.connect(InetAddress.getByName(m_strServerName), m_nServerSessionPort);
      )
   }
}
