/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

//include(`Config_macros.m4')
package mqc.udp;

import java.io.*;
import java.util.*;
import java.lang.*;

__J2SE__(import java.net.*;)
__CLDC__(
import javax.microedition.io.StreamConnection;
import javax.microedition.io.Connector;
import javax.microedition.io.Connection;
import javax.microedition.io.DatagramConnection;
import javax.microedition.io.Datagram;
)

/**
 * class UDPSocket:
 * Base Class for all different types of UDPSocket. Implements
 * the basic send and receive using unreliable UDP datagram socket
 */
public class UDPSocket
{

   /* UDP Data gram socket */
   __J2SE__(
   public DatagramSocket d_socket;
   )
   __CLDC__(
   public DatagramConnection d_socket;
   )

   /* Server info */
   __J2SE__(protected InetAddress server_address;)
   __CLDC__(protected String server_address;)
   protected int server_port;

   /**
    * Function : Constrcutor
    * Purpose  : Takes in the udp socket to be used for communication, 
    * server name & server port
    */
   public UDPSocket(__J2SE__(DatagramSocket socketIn,)
                    __CLDC__(DatagramConnection socketIn,)
                    String receiver, int receiver_port)
   {
      /* Set the UDPSocket */
      d_socket = socketIn;

      __J2SE__(
      try
      {
         server_address = InetAddress.getByName(receiver);
         d_socket.connect( server_address, receiver_port);
      }
      catch (UnknownHostException uhe)
      {
         System.out.println("**Error: could not set host name " + receiver);
      }
      )

      /* Set server information */
      this.server_port = receiver_port;

      __CLDC__(server_address = receiver;)
   }

   /**
    * Function : send
    * Purpose  : To send the data inform of udp packet
    */
   public void send(byte [] data, int size)
   {

      __J2SE__(
      /* Send the packet */
      try
      {
         d_socket.send( new DatagramPacket(data, size, server_address, server_port ));
      }
      catch( Exception e)
      {
         System.out.println("**Error: sending data to socket");
      }
      )

      __CLDC__(
      try
      {
         Datagram sdg = d_socket.newDatagram(size);
         sdg.setData(data, 0, size);
         sdg.setAddress("datagram://" + server_address + ":" + server_port);
         d_socket.send(sdg); //TODO: handle timeout
      }
      catch (Exception ioe)
      {
         System.out.println("**Error: sending data to socket");
         ioe.printStackTrace();
         System.exit(-1);
      }
      )
   }

   /**
    * Function : receive
    * Purpose  : receives a udp packet and passes on the information in data
    * and return value
    */
   public int receive(byte[] data)
   {
      //System.out.println("UDPSocket::receive");

      /* Create a udp packet to receive in the information */
      __J2SE__(DatagramPacket rdg = new DatagramPacket(data, data.length);)
      __CLDC__(Datagram rdg = null; )
      try
      {
         /* Receive the packet */
         __CLDC__(rdg = d_socket.newDatagram(data, data.length);)
         d_socket.receive(rdg);
      }
      catch(IOException e)
      {
         System.out.println("Error receiving data from socket");
         e.printStackTrace();
         return 0;
      }

      return rdg.getLength();
   }

   public void disconnect()
   {
      System.out.println("**Warning not implemented yet!!");
   }
}
